<?php
namespace App\JsonRpc;

use App\Model\Department;
use App\Model\District;
use App\Model\LetterOfComplaint;
use App\Model\LetterType;
use App\Model\LevelUser;
use App\Model\UserLevel;
use App\Model\TemplateClass;
use App\Model\Template;
use App\Model\WebsiteTemplate;
use App\Model\WebsiteTemplateInfo;
use App\Model\Sector;
use App\Model\Component;
use App\Tools\Result;
use Hyperf\RpcServer\Annotation\RpcService;
use Hyperf\Support\Collection;
use App\Service\MinioService;


#[RpcService(name: "PublicRpcService", protocol: "jsonrpc-http", server: "jsonrpc-http")]
class PublicRpcService implements PublicRpcServiceInterface
{

    /**
     * @param array $data
     * @return array
     */
    public function getDistrictList(array $data): array
    {
        $where = [];
        if (isset($data['keyWord'])) {
            $where = [
                ['name', 'like', '%' . $data['keyWord'] . '%'],
            ];
        }
        $result = [];
        if (isset($data['pageSize'])) {
            $rep = District::where($where)->limit($data['pageSize'])->offset(($data['page'] - 1) * $data['pageSize'])->orderBy("code", "asc")->get();
            $count = District::where($where)->count();
            $result = [
                'rows' => $rep,
                'count' => $count,

            ];
        } else {
            $result = District::where($data)->orderBy("code", "asc")->get();
        }
        return $result ? Result::success($result) : Result::error("没有查到数据");
    }

    /**
     * @param array $data
     * @return array
     */
    public function getUserLevelList(array $data): array
    {
        $where = [];
        if (isset($data['keyWord'])) {
            $where = [
                ['name', 'like', '%' . $data['keyWord'] . '%'],
            ];
        }
        $result = [];
        if (isset($data['pageSize'])) {
            $rep = UserLevel::where($where)->limit($data['pageSize'])->offset(($data['page'] - 1) * $data['pageSize'])->orderBy("id", "asc")->get();
            $count = UserLevel::where($where)->count();
            $result = [
                'rows' => $rep,
                'count' => $count,
            ];
        } else {
            $result = UserLevel::orderBy("id", "asc")->get();
        }
        return $result ? Result::success($result) : Result::error("没有查到数据");
    }

    /**
     * 添加用户等级
     * @param array $data
     * @return array
     */
    public function addUserLevel(array $data): array
    {
        LevelUser::insertGetId($data);
        return Result::success([]);
    }
    /**
     * 更新等级
     * @param array $data
     * @return array
     */
    public function updateUserLevel(array $data): array
    {

        $result = LevelUser::where(['id' => $data['id']])->update($data);
        if ($result) {
            return Result::success($result);
        }
        return Result::error("更新失败");
    }

    /**
     * 删除等级
     * @param array $data
     * @return array
     */
    public function delUserLevel(array $data): array
    {

        $result = LevelUser::where(['id' => $data['id']])->delete();
        if ($result) {
            return Result::success($result);
        }
        return Result::error("删除失败");
    }

    /**
     * 查询投诉举报信息
     * @param array $data
     * @return array
     */
    public function getLetterOfComplaint(array $data = []): array
    {
        var_dump("====");
        $where = [];

        if (isset($data['user_id']) && !empty($data['user_id'])) {
            array_push($where, ['letter_of_complaint.user_id', '=', $data['user_id']]);
        }
        if (isset($data['nature']) && !empty($data['nature'])) {
            array_push($where, ['letter_of_complaint.nature', '=', $data['nature']]);
        }

        if (isset($data['nature_level0']) && !empty($data['nature_level0'])) {
            array_push($where, ['letter_of_complaint.nature_level0', '=', $data['nature_level0']]);
        }

        if (isset($data['status']) && !empty($data['status'])) {
            array_push($where, ['letter_of_complaint.status', '=', $data['status']]);
        }
        $result = [];
        if (isset($data['pageSize'])) {
            $rep = LetterOfComplaint::where($where)
                ->leftJoin("letter_type as type_a", "letter_of_complaint.nature", "type_a.id")
                ->leftJoin("letter_type as type_c", "letter_of_complaint.nature_level0", "type_c.id")
                ->leftJoin("letter_type as type_b", "letter_of_complaint.nature_level1", "type_b.id")
                ->leftJoin("letter_type as type_e", "letter_of_complaint.nature_level3", "type_e.id")
                ->leftJoin("letter_type as type_d", "letter_of_complaint.status", "type_d.id")

                ->select("letter_of_complaint.*",
                    "type_a.type_name as nature_name",
                    "type_b.type_name as nature_name1",
                    "type_c.type_name as nature_name0",
                    "type_d.type_name as status_name",
                    "type_e.type_name as nature_name3")
                ->limit($data['pageSize'])->offset(($data['page'] - 1) * $data['pageSize'])->orderBy("letter_of_complaint.id", "desc")->get();
            $count = LetterOfComplaint::where($where)->count();
            if ($rep) {
                foreach ($rep as $val) {
                    if ($val['judgment']) {
                        $val['judgment'] = json_decode($val['judgment']);
                    }
                    if ($val['audio_and_video']) {
                        $val['audio_and_video'] = json_decode($val['audio_and_video']);
                    }
                    if ($val['contract']) {
                        $val['contract'] = json_decode($val['contract']);
                    }
                    if ($val['qualifications']) {
                        $val['qualifications'] = json_decode($val['qualifications']);
                    }
                }
            }
            $result = [
                'rows' => $rep,
                'count' => $count,
            ];
        } else {
            $result = LetterOfComplaint::where($where)
                ->leftJoin("letter_type as type_a", "letter_of_complaint.nature", "type_a.id")
                ->leftJoin("letter_type as type_c", "letter_of_complaint.nature_level0", "type_c.id")
                ->leftJoin("letter_type as type_b", "letter_of_complaint.nature_level1", "type_b.id")
                ->leftJoin("letter_type as type_e", "letter_of_complaint.nature_level3", "type_e.id")
                ->leftJoin("letter_type as type_d", "letter_of_complaint.status", "type_d.id")

                ->select("letter_of_complaint.*",
                    "type_a.type_name as nature_name",
                    "type_b.type_name as nature_name1",
                    "type_c.type_name as nature_name0",
                    "type_d.type_name as status_name",
                    "type_e.type_name as nature_name3")
                ->orderBy("letter_of_complaint.id", "desc")->get();
        }
        return $result ? Result::success($result) : Result::error("没有查到数据");
    }

    /**
     * 添加投诉举报信息
     * @param array $data
     * @return array
     */
    public function addLetterOfComplaint(array $data): array
    {
        $data['judgment'] = $data['judgment'] ? json_encode($data['judgment']) : '';
        $data['audio_and_video'] = $data['audio_and_video'] ? json_encode($data['audio_and_video']) : '';
        $data['contract'] = $data['contract'] ? json_encode($data['contract']) : '';
        $data['qualifications'] = $data['qualifications'] ? json_encode($data['qualifications']) : '';
        unset($data['id']);
        $result = LetterOfComplaint::insertGetId($data);
        if (empty($result)) {
            return Result::error("创建失败", 0);
        } else {
            return Result::success(["id" => $result]);
        }
    }

    /**
     * 用户端更新投诉举报
     * @param array $data
     * @return array
     */
    public function userUpLetterOfComplaint(array $data): array
    {
        $data['judgment'] = $data['judgment'] ? json_encode($data['judgment']) : '';
        $data['audio_and_video'] = $data['audio_and_video'] ? json_encode($data['audio_and_video']) : '';
        $data['contract'] = $data['contract'] ? json_encode($data['contract']) : '';
        $data['qualifications'] = $data['qualifications'] ? json_encode($data['qualifications']) : '';
        $result = LetterOfComplaint::where(['id' => $data['id']])->update($data);
        if (empty($result)) {
            return Result::error("创建失败", 0);
        } else {
            return Result::success(["id" => $result]);
        }
    }
    /**
     * 管理后台更新投诉举报信息
     * @param array $data
     * @return array
     */
    public function upLetterOfComplaint(array $data): array
    {
        var_dump("admin:", $data);
        $where = [
            'id' => $data['id'],
        ];

        $filtered_array = array_filter($data, function ($value) {
            return $value !== "" && $value !== null && $value !== false && !is_array($value) || !empty($value);
        });
        $filtered_array['judgment'] = isset($filtered_array['judgment']) ? json_encode($filtered_array['judgment']) : '';
        $filtered_array['audio_and_video'] = isset($filtered_array['audio_and_video']) ? json_encode($filtered_array['audio_and_video']) : '';
        $filtered_array['contract'] = isset($filtered_array['contract']) ? json_encode($filtered_array['contract']) : '';
        $filtered_array['qualifications'] = isset($filtered_array['qualifications']) ? json_encode($filtered_array['qualifications']) : '';
        unset($filtered_array['nature_name']);
        unset($filtered_array['type_name']);
        unset($filtered_array['nature_level_name']);
        unset($filtered_array['status_name']);
        unset($filtered_array['is_admin']);
        unset($filtered_array['type_level_name']);
        $result = LetterOfComplaint::where($where)->update($filtered_array);
        if ($result) {
            return Result::success($result);
        }
        return Result::error("更新失败", 0);
    }

    /**
     * 查询投诉举报记录
     * @param array $data
     * @return array
     */
    public function getLetterOfComplaintInfo(array $data): array
    {
        $where = [
            'letter_of_complaint.id' => $data['id'],
        ];
        if (isset($data['user_id']) && !empty($data['user_id'])) {
            array_push($where, ['letter_of_complaint.user_id', '=', $data['user_id']]);
        }
        $result = LetterOfComplaint::where($where)
            ->leftJoin("letter_type as type_a", "letter_of_complaint.nature", "type_a.id")
            ->leftJoin("letter_type as type_c", "letter_of_complaint.nature_level0", "type_c.id")
            ->leftJoin("letter_type as type_b", "letter_of_complaint.nature_level1", "type_b.id")
            ->leftJoin("letter_type as type_e", "letter_of_complaint.nature_level3", "type_e.id")
            ->leftJoin("letter_type as type_d", "letter_of_complaint.status", "type_d.id")

            ->select("letter_of_complaint.*",
                "type_a.type_name as nature_name",
                "type_b.type_name as nature_name1",
                "type_c.type_name as nature_name0",
                "type_d.type_name as status_name",
                "type_e.type_name as nature_name3")
            ->first();
        return Result::success($result);
    }

    /**
     * 删除投诉举报信息
     * @param array $data
     * @return array
     */
    public function delLetterOfComplaint(array $data): array
    {
        $result = LetterOfComplaint::when($data,function ($query) use ($data){
            if(isset($data['id']) && !empty($data['id'])){
                $query->where(['id'=>$data['id']]);
            }
            if(isset($data['user_id']) && !empty($data['user_id'])){
                $query->where(['user_id'=>$data['user_id']]);
            }
        })->delete();
        if (empty($result)) {
            return Result::error("删除失败", 0);
        } else {
            return Result::success();
        }
    }
    /**
     * 获取举报信息类型
     * @param array $data
     * @return array
     */
    public function getLetterType(array $data): array
    {
        $where = [];
        if (isset($data['type'])) {
            array_push($where, ['type', '=', $data['type']]);
        }
        if (isset($data['pid']) && $data['pid'] > 0) {
            array_push($where, ['pid', '=', $data['pid']]);
        }
        $result = LetterType::where($where)->orderBy('sort', 'asc')->get();
        return $result ? Result::success($result) : Result::error("没有查到数据");
    }
    /**
     * 更新举报类型
     * @param array $data
     * @return array
     */
    public function upLetterType(array $data): array
    {
        return [];
    }

    /**
     * 添加举报类型
     * @param array $data
     * @return array
     */
    public function addLetterType(array $data): array
    {
        $result = LetterType::insertGetId($data);
        if (empty($result)) {
            return Result::error("创建失败", 0);
        } else {
            return Result::success(["id" => $result]);
        }
    }

    /**
     * 删除举报类型
     * @param array $data
     * @return array
     */
    public function delLetterType(array $data): array
    {
        $result = LetterType::where('id', $data['id'])->delete();
        if (empty($result)) {
            return Result::error("删除失败", 0);
        } else {
            return Result::success();
        }
    }

    /**
     * 检测是否已经被接案
     * @param array $data
     * @return array
     */
    public function checkMeasure(array $data): array
    {
        $where = [
            'id' => $data['id'],
        ];
        $letterOfComplaintInfo = LetterOfComplaint::where($where)->first();
        var_dump("查询数据：", $letterOfComplaintInfo['admin_id'], $data['user_id']);
        //操作人和当前登陆用户id 相等说明是当前人接收的案件
        if (($letterOfComplaintInfo['admin_id'] == $data['user_id']) || empty($letterOfComplaintInfo['admin_id'])) {
            return Result::success();
        } else {
            return Result::error("您不能处理其他人已经接过的案件", 0);
        }

    }

    /**
     * 后台获取职能部门
     * @param array $data
     * @return array
     */
    public function getZhinengbumenList(array $data): array
    {

        // 获取分页参数，默认每页 10 条记录
        $page = isset($data['page']) ? (int) $data['page'] : 1;
        $perPage = isset($data['pagesize']) ? (int) $data['pagesize'] : 10;

        // 查询数据并分页
        $query = Department::query();

        // 可以在这里添加更多的查询条件
        if (isset($data['search'])) {
            $query->where('name', 'like', '%' . $data['search'] . '%');
        }

        // 执行分页查询
        $result = $query->paginate($perPage, ['*'], 'page', $page);

        // 返回分页结果
        return Result::success([
            'count' => $result->total(),
            'current_page' => $result->currentPage(),
            'last_page' => $result->lastPage(),
            'pagesize' => $result->perPage(),
            'rows' => $result->items(),
        ]);
    }

    /**
     * 添加获取职能部门
     * @param array $data
     * @return array
     */
    public function addZhinengbumen(array $data): array
    {
        $result = Department::insertGetId($data);
        if (empty($result)) {
            return Result::error("创建失败", 0);
        } else {
            return Result::success(["id" => $result]);
        }
    }
    public function delZhinengbumen(array $data): array
    {
        $result = Department::where('id', $data['id'])->delete();
        if (empty($result)) {
            return Result::error("删除失败", 0);
        } else {
            return Result::success();
        }
    }
    public function getZhinengbumen(array $data): array
    {
        $result = Department::where('id', $data['id'])->first();
        return Result::success($result);
    }
    public function getPidZhinengbumen(array $data): array
    {
        if (empty($data['pid'])) {
            $data['pid'] = 0;
        }

        $result = Department::where('pid', $data['pid'])->get();
        return Result::success($result);
    }
    public function modZhinengbumen(array $data): array
    {
        $result = Department::where('id', $data['id'])->update($data);
        if (empty($result)) {
            return Result::error("修改失败", 0);
        } else {
            return Result::success();
        }
    }

    /**
     * 查询职能列表
     * @param array $data
     * @return array
     */
    public function getDepartment(array $data): array
    {
        $where = [];
        if(isset($data['pid'])){
            $where  = [
                'pid'=>$data['pid']??0
            ];
        }
        $result = Department::when(!empty($where),function ($query) use ($where){
            $query->where($where);
        })->orderBy("sort","desc")->get();
        if (empty($result)) {
            return Result::error("查询失败", 0);
        } else {
            return Result::success($result);
        }
    }

    /**
     * 获取getTemplateClass
     * @param array $data
     * @return array
     */
    public function getTemplateClass(array $data): array
    {
        $result = TemplateClass::get();
        return Result::success($result);
    }
    public function getTemplateList(array $data): array
    {
        $where = [];
        if (!empty($data['template_class_id'])) {
            $where['template_class_id'] = $data['template_class_id'];
        }
        if (!empty($data['template_name'])) {
            $where['template_name'] = $data['template_name'];
        }
        $result = Template::where($where)
            ->leftJoin('template_class', 'template.template_class_id', 'template_class.id')
            ->select('template.*', 'template_class.name as template_class_name')
            ->orderBy('template.id', 'desc')
            ->paginate($data['page_size'], ['*'], 'mypage_name', $data['page']);
        // 使用 items 方法获取分页数据
        // $items = collect($result->items());
        // $items->each(function ($item) {
        //     $item['template_content'] = json_decode($item['template_content'], true);
        // });
        return Result::success($result);
    }
    public function getTemplateInfo(array $data): array
    {
        $result = Template::where('template.id', $data['id'])
            ->leftJoin('template_class', 'template.template_class_id', 'template_class.id')
            ->select('template.*', 'template_class.name as template_class_name')
            ->first();
        return Result::success($result);
    }
    public function addTemplate(array $data): array
    {
        var_dump($data);
        unset($data['user_id']);
        $result = Template::insertGetId($data);
        if ($result) {
            $returData = Template::where('id', $result)->first();
            return Result::success($returData);
        } else {
            return Result::error("添加失败", 0);
        }
    }
    public function delTemplate(array $data): array
    {
        $result = Template::where('id', $data['id'])->delete();
        var_dump($result, '-------------------delete');
        if ($result) {
            return Result::success($result);
        } else {
            return Result::error("删除失败", 0);
        }
    }
    public function updateTemplate(array $data): array
    {
        unset($data['user_id']);
        $result = Template::where('id', $data['id'])->update($data);
        var_dump($result, '-------------------update');
        if (!$result) {
            return Result::error("更新失败", 0);
        } else {
            return Result::success('更新成功');
        }
    }
    public function getSectorList(array $data): array
    {
        $where = [];
        if (!empty($data['template_class_id'])) {
            $where['template_class.id'] = $data['template_class_id'];
        }
        if (!empty($data['template_class_name'])) {
            $where[] = ['template_class.name', 'like', '%' . $data['template_class_name'] . '%'];
        }


        if (!empty($data['sector_name'])) {
            if (!empty($data['sector_name'])) {
                // $where['sector_name'] = $data['sector_name'];
                $where[] = ['sector.sector_name', 'like', '%' . $data['sector_name'] . '%'];
            }
        }

        $result = Sector::where($where)
            ->leftJoin('template', 'template.id', '=', 'sector.template_id')
            ->leftJoin('template_class', 'template_class.id', '=', 'sector.template_id') // 添加这一行
            ->select('sector.*', 'sector.sector_name', 'template.template_name', 'template_class.name as template_class_name', 'template_class.id as template_class_id') // 修改这一行
            ->orderBy('sector.id', 'desc')
            ->paginate($data['page_size'], ['*'], 'mypage_name', $data['page']);

        return Result::success($result);
    }
    public function getSectorInfo(array $data): array
    {
        $where = [];
        $where[] = ['sector.id', '=', $data['id']];
        $result = Sector::where($where)
            ->leftJoin('template', 'template.id', '=', 'sector.template_id')
            ->leftJoin('template_class', 'template_class.id', '=', 'template.template_class_id') // 添加这一行
            ->select('sector.*', 'template.template_name', 'template_class.name as template_class_name', 'template_class.id as template_class_id') // 修改这一行
            ->orderBy('sector.id', 'desc')
            ->get();
        return Result::success($result);
    }
    public function addSector(array $data): array
    {
        unset($data['user_id']);
        // $data['page_type'] = json_encode($data['page_type']);
        $result = Sector::insertGetId($data);
        return Result::success();
    }
    public function delSector(array $data): array
    {
        $result = Sector::where('id', $data['id'])->delete();
        if ($result == 1) {
            return Result::success('删除成功');
        } else {
            return Result::error('删除失败');
        }
    }
    public function updateSector(array $data): array
    {
        unset($data['user_id']);
        $result = Sector::where('id', $data['id'])->update($data);
        if ($result == 1) {
            return Result::success('修改成功');
        } else {
            return Result::error('修改失败');
        }
    }
    public function getComponentList(array $data): array
    {
        var_dump($data, '---------');
        $where = [];
        // $where[] = ['sector.id', '=', $data['id']];
        if (!empty($data['template_class_id'])) {
            $where['template_class.id'] = $data['template_class_id'];
        }
        if (!empty($data['component_name'])) {
            $where['component.component_name'] = $data['component_name'];
        }
        if (!empty($data['sector_id'])) {
            $where['sector.id'] = $data['sector_id'];
        };

        $result = Component::where($where)
            ->leftJoin('template', 'template.id', '=', 'component.template_id')
            ->leftJoin('template_class', 'template_class.id', '=', 'template.template_class_id') // 添加这一行
            ->leftJoin('sector', 'sector.id', '=', 'component.sector_id')
            ->select('template_class.name as template_class_name', 'template.template_name as template_name', 'template_class.id as template_class_id', 'sector.sector_name as sector_name', 'component.*', 'sector.id as sector_id') // 修改这一行)
            ->orderBy('sector.updated_at', 'desc')
            ->orderBy('sector.created_at', 'desc')
            ->paginate($data['page_size'], ['*'], 'mypage_name', $data['page']);
        return Result::success($result);
    }
    public function getComponentInfo(array $data): array
    {
        $where = [];
        $result = Component::where($where)
            ->leftJoin('template', 'template.id', '=', 'component.template_id')
            ->leftJoin('template_class', 'template_class.id', '=', 'template.template_class_id') // 添加这一行
            ->leftJoin('sector', 'sector.id', '=', 'component.sector_id')
            ->select('template_class.name as template_class_name', 'template.template_name as template_name', 'sector.sector_name as sector_name', 'component.*')
            ->get();
        return Result::success($result);
    }
    public function addComponent(array $data): array
    {
        unset($data['user_id']);
        $result = Component::insertGetId($data);
        if ($result) {
            return Result::success($result);
        } else {
            return Result::error('添加失败');
        }
    }
    public function delComponent(array $data): array
    {
        $result = Component::where('id', $data['id'])->delete();
        return Result::success($result);
    }
    public function updateComponent(array $data): array
    {
        $result = Component::where('id', $data['id'])->update($data);
        return Result::success($result);
    }
    public function getWebsiteTemplateList(array $data): array
    {
        $where = [];

        $result = WebsiteTemplateInfo::where($where)
            ->leftJoin('website', 'website_template_info.website_id', '=', 'website.id')
            ->select('website_template_info.*', 'website.website_name')
            ->orderBy('website_template_info.id', 'desc')
            ->paginate($data['page_size'], ['*'], 'page', $data['page']);
        if ($result) {
            return Result::success($result);
        } else {
            return Result::error('暂无数据');
        }
    }
    public function getWebsiteTemplateInfo(array $data)
    {
        $where = [];
        if (isset($data['id'])) {
            $where[] = ['id', '=', $data['id']];
        }
        $result = WebsiteTemplateInfo::where($where)
            ->leftJoin('website', 'website_template_info.website_id', '=', 'website.id')
            ->leftJoin('website_template', 'website_template_info.template_id', '=', 'website_template.id')
            ->select('website_template_info.*', 'website.website_name')
            ->get();
        if ($result) {
            return Result::success($result);
        } else {
            return Result::error('暂无数据');
        }

    }


    /**
     * 获取所有的buckets
     * @param array $data
     * @return array
     */
    public function getBuckets(array $data) :array
    {
        $result = new MinioService();
        // 调用服务层的方法获取存储桶列表
        $bucketsResponse =$result->listBuckets();
        // 直接返回服务层生成的响应
        return Result::success($bucketsResponse['data']);
    }

    /**
     * 上传文件
     * @param array $data
     * @return array
     */
    public function uploadFile(array $data) :array
    {
        $result = new MinioService();
        $rep = $result->uploadFile($data);
        if($rep['code']==200){
            return Result::success($rep['data']);
        }else{
            return Result::error("上传失败！");
        }

    }


}
